@@ -35,14 +35,15 @@
 #include <sched.h>
 
 #include "compat-getcpu.h"
-#include "urcu/wfcqueue.h"
-#include "urcu-call-rcu.h"
-#include "urcu-pointer.h"
-#include "urcu/list.h"
-#include "urcu/futex.h"
-#include "urcu/tls-compat.h"
-#include "urcu/ref.h"
+#include <urcu/wfcqueue.h>
+#include <urcu/call-rcu.h>
+#include <urcu/pointer.h>
+#include <urcu/list.h>
+#include <urcu/futex.h>
+#include <urcu/tls-compat.h>
+#include <urcu/ref.h>
 #include "urcu-die.h"
+#include "urcu-utils.h"
 
 #define SET_AFFINITY_CHECK_PERIOD		(1U << 8)	/* 256 */
 #define SET_AFFINITY_CHECK_PERIOD_MASK		(SET_AFFINITY_CHECK_PERIOD - 1)
@@ -464,6 +465,8 @@ struct call_rcu_data *get_cpu_call_rcu_data(int cpu)
 		return NULL;
 	return rcu_dereference(pcpu_crdp[cpu]);
 }
+URCU_ATTR_ALIAS(urcu_stringify(get_cpu_call_rcu_data))
+struct call_rcu_data *alias_get_cpu_call_rcu_data();
 
 /*
  * Return the tid corresponding to the call_rcu thread whose
@@ -474,6 +477,8 @@ pthread_t get_call_rcu_thread(struct call_rcu_data *crdp)
 {
 	return crdp->tid;
 }
+URCU_ATTR_ALIAS(urcu_stringify(get_call_rcu_thread))
+pthread_t alias_get_call_rcu_thread();
 
 /*
  * Create a call_rcu_data structure (with thread) and return a pointer.
@@ -488,6 +493,8 @@ static struct call_rcu_data *__create_call_rcu_data(unsigned long flags,
 	return crdp;
 }
 
+URCU_ATTR_ALIAS(urcu_stringify(create_call_rcu_data))
+struct call_rcu_data *alias_create_call_rcu_data();
 struct call_rcu_data *create_call_rcu_data(unsigned long flags,
 					   int cpu_affinity)
 {
@@ -544,6 +551,8 @@ int set_cpu_call_rcu_data(int cpu, struct call_rcu_data *crdp)
 	call_rcu_unlock(&call_rcu_mutex);
 	return 0;
 }
+URCU_ATTR_ALIAS(urcu_stringify(set_cpu_call_rcu_data))
+int alias_set_cpu_call_rcu_data();
 
 /*
  * Return a pointer to the default call_rcu_data structure, creating
@@ -564,6 +573,8 @@ struct call_rcu_data *get_default_call_rcu_data(void)
 	call_rcu_unlock(&call_rcu_mutex);
 	return default_call_rcu_data;
 }
+URCU_ATTR_ALIAS(urcu_stringify(get_default_call_rcu_data))
+struct call_rcu_data *alias_get_default_call_rcu_data();
 
 /*
  * Return the call_rcu_data structure that applies to the currently
@@ -591,6 +602,8 @@ struct call_rcu_data *get_call_rcu_data(void)
 
 	return get_default_call_rcu_data();
 }
+URCU_ATTR_ALIAS(urcu_stringify(get_call_rcu_data))
+struct call_rcu_data *alias_get_call_rcu_data();
 
 /*
  * Return a pointer to this task's call_rcu_data if there is one.
@@ -600,6 +613,8 @@ struct call_rcu_data *get_thread_call_rcu_data(void)
 {
 	return URCU_TLS(thread_call_rcu_data);
 }
+URCU_ATTR_ALIAS(urcu_stringify(get_thread_call_rcu_data))
+struct call_rcu_data *alias_get_thread_call_rcu_data();
 
 /*
  * Set this task's call_rcu_data structure as specified, regardless
@@ -616,6 +631,8 @@ void set_thread_call_rcu_data(struct call_rcu_data *crdp)
 {
 	URCU_TLS(thread_call_rcu_data) = crdp;
 }
+URCU_ATTR_ALIAS(urcu_stringify(set_thread_call_rcu_data))
+void alias_set_thread_call_rcu_data();
 
 /*
  * Create a separate call_rcu thread for each CPU.  This does not
@@ -667,6 +684,8 @@ int create_all_cpu_call_rcu_data(unsigned long flags)
 	}
 	return 0;
 }
+URCU_ATTR_ALIAS(urcu_stringify(create_all_cpu_call_rcu_data))
+int alias_create_all_cpu_call_rcu_data();
 
 /*
  * Wake up the call_rcu thread corresponding to the specified
@@ -714,6 +733,7 @@ void call_rcu(struct rcu_head *head,
 	_call_rcu(head, func, crdp);
 	_rcu_read_unlock();
 }
+URCU_ATTR_ALIAS(urcu_stringify(call_rcu)) void alias_call_rcu();
 
 /*
  * Free up the specified call_rcu_data structure, terminating the
@@ -769,6 +789,8 @@ void call_rcu_data_free(struct call_rcu_data *crdp)
 
 	free(crdp);
 }
+URCU_ATTR_ALIAS(urcu_stringify(call_rcu_data_free))
+void alias_call_rcu_data_free();
 
 /*
  * Clean up all the per-CPU call_rcu threads.
@@ -809,6 +831,16 @@ void free_all_cpu_call_rcu_data(void)
 	}
 	free(crdp);
 }
+#ifdef RCU_QSBR
+/* ABI6 has a non-namespaced free_all_cpu_call_rcu_data for qsbr */
+#undef free_all_cpu_call_rcu_data
+URCU_ATTR_ALIAS("urcu_qsbr_free_all_cpu_call_rcu_data")
+void free_all_cpu_call_rcu_data();
+#define free_all_cpu_call_rcu_data urcu_qsbr_free_all_cpu_call_rcu_data
+#else
+URCU_ATTR_ALIAS(urcu_stringify(free_all_cpu_call_rcu_data))
+void alias_free_all_cpu_call_rcu_data();
+#endif
 
 static
 void free_completion(struct urcu_ref *ref)
@@ -900,6 +932,8 @@ online:
 	if (was_online)
 		rcu_thread_online();
 }
+URCU_ATTR_ALIAS(urcu_stringify(rcu_barrier))
+void alias_rcu_barrier();
 
 /*
  * Acquire the call_rcu_mutex in order to ensure that the child sees
@@ -928,6 +962,8 @@ void call_rcu_before_fork(void)
 			(void) poll(NULL, 0, 1);
 	}
 }
+URCU_ATTR_ALIAS(urcu_stringify(call_rcu_before_fork))
+void alias_call_rcu_before_fork();
 
 /*
  * Clean up call_rcu data structures in the parent of a successful fork()
@@ -950,6 +986,8 @@ void call_rcu_after_fork_parent(void)
 		atfork->after_fork_parent(atfork->priv);
 	call_rcu_unlock(&call_rcu_mutex);
 }
+URCU_ATTR_ALIAS(urcu_stringify(call_rcu_after_fork_parent))
+void alias_call_rcu_after_fork_parent();
 
 /*
  * Clean up call_rcu data structures in the child of a successful fork()
@@ -997,6 +1035,8 @@ void call_rcu_after_fork_child(void)
 		call_rcu_data_free(crdp);
 	}
 }
+URCU_ATTR_ALIAS(urcu_stringify(call_rcu_after_fork_child))
+void alias_call_rcu_after_fork_child();
 
 void urcu_register_rculfhash_atfork(struct urcu_atfork *atfork)
 {
@@ -1007,6 +1047,8 @@ void urcu_register_rculfhash_atfork(struct urcu_atfork *atfork)
 end:
 	call_rcu_unlock(&call_rcu_mutex);
 }
+URCU_ATTR_ALIAS(urcu_stringify(urcu_register_rculfhash_atfork))
+void alias_urcu_register_rculfhash_atfork();
 
 void urcu_unregister_rculfhash_atfork(struct urcu_atfork *atfork)
 {
@@ -1017,3 +1059,5 @@ void urcu_unregister_rculfhash_atfork(struct urcu_atfork *atfork)
 end:
 	call_rcu_unlock(&call_rcu_mutex);
 }
+URCU_ATTR_ALIAS(urcu_stringify(urcu_unregister_rculfhash_atfork))
+void alias_urcu_unregister_rculfhash_atfork();
